@@ -24,7 +24,6 @@ import (
 	"github.com/rootless-containers/rootlesskit/pkg/port/builtin"
 	"github.com/rootless-containers/rootlesskit/pkg/port/portutil"
 	slirp4netns_port "github.com/rootless-containers/rootlesskit/pkg/port/slirp4netns"
-	"github.com/rootless-containers/rootlesskit/pkg/port/socat"
 	"github.com/rootless-containers/rootlesskit/pkg/version"
 )
 
@@ -60,119 +59,142 @@ Examples:
   # create a network namespace with slirp4netns, and expose 80/tcp on the namespace as 8080/tcp on the host
   rootlesskit --copy-up=/etc --net=slirp4netns --disable-host-loopback --port-driver=builtin -p 127.0.0.1:8080:80/tcp bash
 
-Note: RootlessKit requires /etc/subuid and /etc/subgid to be configured by the real root user.`
+Note: RootlessKit requires /etc/subuid and /etc/subgid to be configured by the real root user.
+See https://rootlesscontaine.rs/getting-started/common/ .
+`
 	app.Flags = []cli.Flag{
-		&cli.BoolFlag{
+		Categorize(&cli.BoolFlag{
 			Name:        "debug",
 			Usage:       "debug mode",
 			Destination: &debug,
-		},
-		&cli.StringFlag{
+		}, CategoryMisc),
+		Categorize(&cli.StringFlag{
 			Name:  "state-dir",
 			Usage: "state directory",
-		},
-		&cli.StringFlag{
+		}, CategoryState),
+		Categorize(&cli.StringFlag{
 			Name:  "net",
 			Usage: "network driver [host, slirp4netns, vpnkit, lxc-user-nic(experimental)]",
 			Value: "host",
-		},
-		&cli.StringFlag{
+		}, CategoryNetwork),
+		Categorize(&cli.StringFlag{
 			Name:  "slirp4netns-binary",
 			Usage: "path of slirp4netns binary for --net=slirp4netns",
 			Value: "slirp4netns",
-		},
-		&cli.StringFlag{
+		}, CategorySlirp4netns),
+		Categorize(&cli.StringFlag{
 			Name:  "slirp4netns-sandbox",
 			Usage: "enable slirp4netns sandbox (experimental) [auto, true, false] (the default is planned to be \"auto\" in future)",
 			Value: "false",
-		},
-		&cli.StringFlag{
+		}, CategorySlirp4netns),
+		Categorize(&cli.StringFlag{
 			Name:  "slirp4netns-seccomp",
 			Usage: "enable slirp4netns seccomp (experimental) [auto, true, false] (the default is planned to be \"auto\" in future)",
 			Value: "false",
-		},
-		&cli.StringFlag{
+		}, CategorySlirp4netns),
+		Categorize(&cli.StringFlag{
 			Name:  "vpnkit-binary",
 			Usage: "path of VPNKit binary for --net=vpnkit",
 			Value: "vpnkit",
-		},
-		&cli.StringFlag{
+		}, CategoryVPNKit),
+		Categorize(&cli.StringFlag{
 			Name:  "lxc-user-nic-binary",
 			Usage: "path of lxc-user-nic binary for --net=lxc-user-nic",
 			Value: lxcUserNicBin(),
-		},
-		&cli.StringFlag{
+		}, CategoryLXCUserNic),
+		Categorize(&cli.StringFlag{
 			Name:  "lxc-user-nic-bridge",
 			Usage: "lxc-user-nic bridge name",
 			Value: "lxcbr0",
-		},
-		&cli.IntFlag{
+		}, CategoryLXCUserNic),
+		Categorize(&cli.IntFlag{
 			Name:  "mtu",
 			Usage: "MTU for non-host network (default: 65520 for slirp4netns, 1500 for others)",
 			Value: 0, // resolved into 65520 for slirp4netns, 1500 for others
-		},
-		&cli.StringFlag{
+		}, CategoryNetwork),
+		Categorize(&cli.StringFlag{
 			Name:  "cidr",
 			Usage: "CIDR for slirp4netns network (default: 10.0.2.0/24)",
-		},
-		&cli.StringFlag{
+		}, CategoryNetwork),
+		Categorize(&cli.StringFlag{
 			Name:  "ifname",
 			Usage: "Network interface name (default: tap0 for slirp4netns and vpnkit, eth0 for lxc-user-nic)",
-		},
-		&cli.BoolFlag{
+		}, CategoryNetwork),
+		Categorize(&cli.BoolFlag{
 			Name:  "disable-host-loopback",
 			Usage: "prohibit connecting to 127.0.0.1:* on the host namespace",
-		},
-		&cli.StringSliceFlag{
+		}, CategoryNetwork),
+		Categorize(&cli.BoolFlag{
+			Name:  "ipv6",
+			Usage: "enable IPv6 routing. Unrelated to port forwarding. Only supported for slirp4netns. (experimental)",
+		}, CategoryNetwork),
+		Categorize(&cli.StringSliceFlag{
 			Name:  "copy-up",
 			Usage: "mount a filesystem and copy-up the contents. e.g. \"--copy-up=/etc\" (typically required for non-host network)",
-		},
-		&cli.StringFlag{
+		}, CategoryMount),
+		Categorize(&cli.StringFlag{
 			Name:  "copy-up-mode",
 			Usage: "copy-up mode [tmpfs+symlink]",
 			Value: "tmpfs+symlink",
-		},
-		&cli.StringFlag{
+		}, CategoryMount),
+		Categorize(&cli.StringFlag{
 			Name:  "port-driver",
-			Usage: "port driver for non-host network. [none, builtin, slirp4netns, socat(deprecated)]",
+			Usage: "port driver for non-host network. [none, builtin, slirp4netns]",
 			Value: "none",
-		},
-		&cli.StringSliceFlag{
+		}, CategoryPort),
+		Categorize(&cli.StringSliceFlag{
 			Name:    "publish",
 			Aliases: []string{"p"},
 			Usage:   "publish ports. e.g. \"127.0.0.1:8080:80/tcp\"",
-		},
-		&cli.BoolFlag{
+		}, CategoryPort),
+		Categorize(&cli.BoolFlag{
 			Name:  "pidns",
 			Usage: "create a PID namespace",
-		},
-		&cli.BoolFlag{
+		}, CategoryProcess),
+		Categorize(&cli.BoolFlag{
 			Name:  "cgroupns",
 			Usage: "create a cgroup namespace",
-		},
-		&cli.BoolFlag{
+		}, CategoryProcess),
+		Categorize(&cli.BoolFlag{
 			Name:  "utsns",
 			Usage: "create a UTS namespace",
-		},
-		&cli.BoolFlag{
+		}, CategoryProcess),
+		Categorize(&cli.BoolFlag{
 			Name:  "ipcns",
 			Usage: "create an IPC namespace",
-		},
-		&cli.StringFlag{
+		}, CategoryProcess),
+		Categorize(&cli.StringFlag{
 			Name:  "propagation",
 			Usage: "mount propagation [rprivate, rslave]",
 			Value: "rprivate",
-		},
-		&cli.StringFlag{
+		}, CategoryMount),
+		Categorize(&cli.StringFlag{
 			Name:  "reaper",
 			Usage: "enable process reaper. Requires --pidns. [auto,true,false]",
 			Value: "auto",
-		},
-		&cli.StringFlag{
+		}, CategoryProcess),
+		Categorize(&cli.StringFlag{
 			Name:  "evacuate-cgroup2",
 			Usage: "evacuate processes into the specified subgroup. Requires --pidns and --cgroupns",
-		},
+		}, CategoryProcess),
 	}
+	app.CustomAppHelpTemplate = `NAME:
+   {{.Name}}{{if .Usage}} - {{.Usage}}{{end}}
+
+USAGE:
+   {{if .UsageText}}{{.UsageText}}{{else}}{{.HelpName}} {{if .VisibleFlags}}[global options]{{end}}{{if .Commands}} command [command options]{{end}} {{if .ArgsUsage}}{{.ArgsUsage}}{{else}}[arguments...]{{end}}{{end}}{{if .Version}}{{if not .HideVersion}}
+
+VERSION:
+   {{.Version}}{{end}}{{end}}{{if .Description}}
+
+DESCRIPTION:
+   {{.Description | nindent 3 | trim}}{{end}}
+
+OPTIONS:
+` + formatFlags(append(app.Flags,
+		Categorize(cli.HelpFlag, CategoryMisc),
+		Categorize(cli.VersionFlag, CategoryMisc)))
+
 	app.Before = func(context *cli.Context) error {
 		if debug {
 			logrus.SetLevel(logrus.DebugLevel)
@@ -283,6 +305,14 @@ func createParentOpt(clicontext *cli.Context, pipeFDEnvKey, stateDirEnvKey, pare
 		return opt, errors.New("ifname must not contain \"/\"")
 	}
 
+	ipv6 := clicontext.Bool("ipv6")
+	if ipv6 {
+		logrus.Warn("ipv6 is experimental")
+		if s := clicontext.String("net"); s != "slirp4netns" {
+			logrus.Warnf("--ipv6 is discarded for --net=%s", s)
+		}
+	}
+
 	disableHostLoopback := clicontext.Bool("disable-host-loopback")
 	if !disableHostLoopback && clicontext.String("net") != "host" {
 		logrus.Warn("specifying --disable-host-loopback is highly recommended to prohibit connecting to 127.0.0.1:* on the host namespace (requires slirp4netns or VPNKit)")
@@ -356,7 +386,8 @@ func createParentOpt(clicontext *cli.Context, pipeFDEnvKey, stateDirEnvKey, pare
 		default:
 			return opt, errors.Errorf("unsupported slirp4netns-seccomp mode: %q", s)
 		}
-		opt.NetworkDriver, err = slirp4netns.NewParentDriver(&logrusDebugWriter{label: "network/slirp4netns"}, binary, mtu, ipnet, ifname, disableHostLoopback, slirp4netnsAPISocketPath, enableSandbox, enableSeccomp)
+		opt.NetworkDriver, err = slirp4netns.NewParentDriver(&logrusDebugWriter{label: "network/slirp4netns"}, binary, mtu, ipnet, ifname, disableHostLoopback, slirp4netnsAPISocketPath,
+			enableSandbox, enableSeccomp, ipv6)
 		if err != nil {
 			return opt, err
 		}
@@ -394,15 +425,6 @@ func createParentOpt(clicontext *cli.Context, pipeFDEnvKey, stateDirEnvKey, pare
 		if len(clicontext.StringSlice("publish")) != 0 {
 			return opt, errors.Errorf("port driver %q does not support publishing ports", s)
 		}
-	case "socat":
-		logrus.Warn("\"socat\" port driver is deprecated")
-		if opt.NetworkDriver == nil {
-			return opt, errors.New("port driver requires non-host network")
-		}
-		opt.PortDriver, err = socat.NewParentDriver(&logrusDebugWriter{label: "port/socat"})
-		if err != nil {
-			return opt, err
-		}
 	case "slirp4netns":
 		if clicontext.String("net") != "slirp4netns" {
 			return opt, errors.New("port driver requires slirp4netns network")
@@ -495,8 +517,6 @@ func createChildOpt(clicontext *cli.Context, pipeFDEnvKey string, targetCmd []st
 	switch s := clicontext.String("port-driver"); s {
 	case "none":
 		// NOP
-	case "socat":
-		opt.PortDriver = socat.NewChildDriver()
 	case "slirp4netns":
 		opt.PortDriver = slirp4netns_port.NewChildDriver()
 	case "builtin":
